<form action="/login_handler?came_from={{ came_from }}" method="POST">
  <label>Username: <input type="text" name="login"/></label><br/>
  <label>Password: <input type="password" name="password"/></label><br/>
  <input type="submit" value="Login"/><br/>
</form>
